package com.example.sunmi;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.provider.Settings;
import android.util.Log;

import com.example.sunmi.sunmi.BytesUtil;
import com.example.sunmi.sunmi.SunmiBase;

import java.util.HashMap;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** SunmiPlugin */
public class SunmiPlugin implements MethodCallHandler {
  private static final String TAG = SunmiPlugin.class.getCanonicalName();
  private final MethodChannel channel;
  private static final String NAME = "sunmi";
  private Activity activity;
  private Context context;
  private SunmiBase sunmiBase;

  /** Plugin registration. */
  public static void registerWith(Registrar r) {
    final MethodChannel c = new MethodChannel(r.messenger(),NAME);
    c.setMethodCallHandler(new SunmiPlugin(r.context(),r.activity(),c));
  }

  private SunmiPlugin(Context context, Activity activity, MethodChannel channel){
    this.context = context;
    this.activity = activity;
    this.channel = channel;
    this.sunmiBase = new SunmiBase(activity,context);
  }

  @Override
  public void onMethodCall(MethodCall call, Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    } else if(call.method.equals("GetUUID")){
      String uniqueId = Settings.Secure.getString(activity.getContentResolver(),
              Settings.Secure.ANDROID_ID);
      result.success(uniqueId);
    } else if(call.method.equals("bindPrinter")) {
      sunmiBase.bindPrinter();
    } else if(call.method.equals("unbindPrinter")) {
      sunmiBase.unbindPrinter();
    } else if(call.method.equals("printArray")) {
      String json = call.argument("json");
      sunmiBase.printArray(json);
    } else if(call.method.equals("setLineWrap")) {
      int line = call.argument("line");
      sunmiBase.setLineWrap(line);
    } else if(call.method.equals("selfCheckingPrinter")) {
      sunmiBase.selfChecking();
    } else if(call.method.equals("printText")) {
      String text = call.argument("text");
      sunmiBase.printText(text);
    } else if(call.method.equals("printTextWithFont")) {
      String text = call.argument("text");
      String font = call.argument("fontName");
      Float size = call.argument("fontSize");

      sunmiBase.printTextWithFont(text,font,size);
    } else if(call.method.equals("printBitmap")) {
      byte[] bytes = call.argument("bitmap");
      Bitmap bitmap = BitmapFactory.decodeByteArray(bytes,0,bytes.length);

      sunmiBase.printBitmap(bitmap);
    } else if(call.method.equals("printQRCode")) {
      String text = call.argument("text");
      int size = call.argument("moduleSize");
      int level = call.argument("errorLevel");
      byte[] bytes = BytesUtil.getPrintQRCode(text,size,level);

      sunmiBase.sendRAWData(bytes);
    } else if(call.method.equals("printBarcode")) {
      String text = call.argument("text");
      int symbol = call.argument("symbology");
      int height = call.argument("height");
      int width = call.argument("width");
      int textPosition = call.argument("textPosition");
      byte[] bytes = BytesUtil.getPrintBarCode(text,symbol,height,width,textPosition);

      sunmiBase.sendRAWData(bytes);
    } else if(call.method.equals("template02")) {
      String lineHeader = call.argument("lineHeader");
      String lineBody = call.argument("lineBody");
      int fontHeader = call.argument("fontSizeHeader");
      int fontBody = call.argument("fontSizeBody");
      String header = call.argument("header");
      String date = call.argument("date");
      String time = call.argument("time");
      String outlet = call.argument("outlet");
      String salesman = call.argument("salesman");
      String amount = call.argument("amount");
      String paymentType = call.argument("paymentType");
      String paymentDue = call.argument("paymentDue");
      String jsonItems = call.argument("jsonItems");

      sunmiBase.printTemplate02(lineHeader,lineBody,fontHeader,fontBody,header,date,time,outlet,
              salesman,amount,paymentType,paymentDue,jsonItems);
    } else if(call.method.equals("getProperties")) {
        HashMap output = sunmiBase.getProperties();
        result.success(output);
    } else {
      result.notImplemented();
    }
  }
}
