package com.example.sunmi.sunmi;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Build;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Base64;
import android.util.Log;
import android.widget.Toast;

import com.example.sunmi.ThreadPoolManager;
import com.example.sunmi.sunmi.BytesUtil;
import com.example.sunmi.os.*;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;

import woyou.aidlservice.jiuiv5.*;

public class SunmiBase {
    private String TAG = SunmiBase.class.getSimpleName();
    private Activity activity;
    //    private Utils utils;
    private Context context;
    private IWoyouService woyouService;
    private ICallback callback;
    private String LINE_FULL = "______________________________________";
    private int FONT_HEADER = 32;
    private int FONT_BODY = 20;
    private int LEFT_ALIGNMENT = 0;
    private int CENTER_ALIGNMENT = 1;
    private int RIGHT_ALIGNMENT = 2;
    private String URL_GWKBALI = "www.gwkbali.com";

    public SunmiBase(Activity activity,Context context){
        this.activity = activity;
        this.context = context;
//        this.utils = new Utils(activity,context);
        this.callback = null;
    }


    /**
     * =================================== SUNMI AIDL PRINTER ========================================
     */
    // STATE CONNECTION
    public void bindPrinter(){
        callback = new ICallback() {
            @Override
            public void onRunResult(boolean isSuccess) throws RemoteException {
                activity.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Toast.makeText(activity,"Printing...",Toast.LENGTH_LONG);
                    }
                });
            }

            @Override
            public void onReturnString(final String result) throws RemoteException {
                activity.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Toast.makeText(activity,result,Toast.LENGTH_LONG);
                    }
                });
            }

            @Override
            public void onRaiseException(int code, final String msg) throws RemoteException {
                activity.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Toast.makeText(activity,msg,Toast.LENGTH_LONG).show();
                    }
                });
            }

            @Override
            public IBinder asBinder() {
                return null;
            }
        };

        Intent intent = new Intent();
        intent.setPackage("woyou.aidlservice.jiuiv5");
        intent.setAction("woyou.aidlservice.jiuiv5.IWoyouService");
        context.startService(intent);
        context.bindService(intent,connService,Context.BIND_AUTO_CREATE);
    }

    public HashMap getProperties() {
        HashMap<String, Object> output = new HashMap<String, Object>();
        String brand = SystemProperties.get("ro.product.brand");
        String model = SystemProperties.get("ro.product.model");
        String versionname = SystemProperties.get("ro.version.sunmi_versionname");
        String versioncode = SystemProperties.get("ro.version.sunmi_versioncode");
        String serialNumber = Build.VERSION.SDK_INT >= Build.VERSION_CODES.O ? Build.getSerial() : Build.SERIAL;

        output.put("ro.product.brand",brand);
        output.put("ro.product.model",model);
        output.put("ro.version.sunmi_versionname",versionname);
        output.put("ro.version.sunmi_versioncode",versioncode);
            output.put("serial_number",serialNumber);
        return output;
    }

    public void unbindPrinter(){
        this.context.unbindService(connService);
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(activity,"Print service disconnected, success.",Toast.LENGTH_LONG).show();
            }
        });
    }

    private ServiceConnection connService = new ServiceConnection() {
        @Override
        public void onServiceDisconnected(ComponentName name) {
            Toast.makeText(activity,"Print service disconnected, please try again.",Toast.LENGTH_LONG).show();
            woyouService = null;
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e){
                e.printStackTrace();
            }
        }

        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            woyouService = IWoyouService.Stub.asInterface(service);
            try {
                Toast.makeText(activity,"Print service connected. (version "+ woyouService.getServiceVersion()+")",Toast.LENGTH_LONG).show();
            } catch (RemoteException e){
                e.printStackTrace();
            }
        }
    };

    public void initPrinter(){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    woyouService.printerInit(callback);
                } catch (RemoteException e){
                    e.printStackTrace();
                    Log.d(TAG,"Error : "+e.toString());
                }
            }
        });
    }

    /**
     *
     */
    public void selfChecking(){
        ThreadPoolManager.getInstance().executeTask(new Runnable(){
            @Override
            public void run() {
                try {
                    woyouService.printerSelfChecking(callback);
                } catch (RemoteException e) {
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     *
     * @param code 1,2,3,4
     * @return String
     */
    public String getInfo(int code){
        String output = null;
        try {
            if(code == 1){
                output = this.woyouService.getPrinterSerialNo();
            } else if(code == 2){
                output = this.woyouService.getPrinterModal();
            } else if(code == 3){
                output = this.woyouService.getPrinterVersion();
            } else if(code == 4){
                output = this.woyouService.getServiceVersion();
            } else {
                output = "No information found.";
            }
            return output;
        } catch (RemoteException e){
            e.printStackTrace();
            return output;
        }
    }

    /**
     *
     * @param bytes
     */
    public void sendRAWData(final byte[] bytes){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    woyouService.sendRAWData(bytes,callback);
                } catch (RemoteException e){
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     *
     * @param alignment
     */
    public void setAlignment(final int alignment){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    woyouService.setAlignment(alignment, callback);
                } catch (RemoteException e){
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     *
     * @param fontName
     */
    public void setFontName(final String fontName){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    woyouService.setFontName(fontName,callback);
                } catch (RemoteException e){
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     *
     * @param fontSize
     */
    public void setFontSize(final float fontSize){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    woyouService.setFontSize(fontSize,callback);
                } catch (final RemoteException e){
                    activity.runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            e.printStackTrace();
                        }
                    });
                }
            }
        });
    }

    /**
     *
     * @param line
     */
    public void setLineWrap(final int line){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    woyouService.lineWrap(line,callback);
                } catch (RemoteException e){
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     *
     * @param text
     */
    public void printText(final String text){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    woyouService.printText(text,callback);
                } catch (RemoteException e){
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     *
     * @param text
     */
    public void printOriText(final String text){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    woyouService.printOriginalText(text,callback);
                } catch (RemoteException e){
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     *
     * @param text
     * @param fontName
     * @param fontSize
     */
    public void printTextWithFont(final String text, final String fontName, final float fontSize){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    woyouService.printTextWithFont(text,fontName,fontSize,callback);
                } catch (RemoteException e){
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     * Unsupported Arabic
     * @param textArray
     * @param widthArray
     * @param alignArray
     */
    public void printColumnText(final String[] textArray, final int[] widthArray, final int[] alignArray){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    woyouService.printColumnsText(textArray,widthArray,alignArray,callback);
                } catch (RemoteException e){
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     *
     * @param bitmap
     */
    public void printBitmap(final Bitmap bitmap){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    woyouService.printBitmap(bitmap,callback);
                    woyouService.lineWrap(3,callback);
                } catch (RemoteException e){
                    e.printStackTrace();
                }
            }
        });
    }

//  /**
//   *
//   * @param data
//   * @param size
//   */
//  public void printQRCodeWithZxing(final String data, final int size){
//    ThreadPoolManager.getInstance().executeTask(new Runnable() {
//      @Override
//      public void run() {
//        try {
//          byte[] bytes = BytesUtil.getZXingQRCode(data,size);
//          woyouService.sendRAWData(bytes,callback);
//        } catch (RemoteException e){
//          e.printStackTrace();
//        }
//      }
//    });
//  }

    /**
     *
     * @param data
     * @param moduleSize
     * @param errorLevel
     * Data →qr code content.
     * Modulesize →qr code block size, unit: dot, values 4 to 16.
     * Errorlevel qr code error correction level (0-3) :
     * 0→ error correction level L (7%)
     * 1→error correction level M (15%)
     * 2→ error correction level Q (25%)
     * 3→error correction level H (30%)
     */
    public void printQRCode(final String data, final int moduleSize, final int errorLevel){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    woyouService.printQRCode(data,moduleSize,errorLevel,callback);
                } catch (RemoteException e){
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     *
     * @param data
     * @param symbology 0 -> 8
     * @param height 1 - 255, default 162
     * @param width 2 - 6, default 2
     * @param textpos 0 - 3 : 0 →no print text, 1→ above the barcode, 2→ below the barcode, 3→ both
     *                Data →one-dimensional code content.
     * 上海商米科技有限公司打印机开发者文档
     * - 13 -
     * Symbology→ barcode type (0-8) :
     * 0 → UPC-A
     * 1 → UPC-E
     * 2 → JAN13(EAN13)
     * 3 → JAN8(EAN8)
     * 4 → CODE39
     * 5 → ITF
     * 6 → CODABAR
     * 7 → CODE93
     * 8 → CODE128
     * Height →bar code height, value 1-255, default: 162.
     * Width →bar code width, value 2-6, default: 2.
     * TextPosition →text Position (0-3) :
     * 0 →no print text
     * 1→ above the barcode
     * 2→ below the barcode
     * 3→ both
     */
    public void printBarcode(final String data, final int symbology, final int height, final int width, final int textpos){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    woyouService.printBarCode(data, symbology, height, width, textpos, callback);
                } catch (RemoteException e){
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     * TEMPLATE FOR PRINTING QR_CODE
     * @param json
     */
    public void printQRCode_Template_01(final String json){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                String qr;
                try {
                    woyouService.lineWrap(1,callback);
                    woyouService.setAlignment(1,callback);
                    try {
                        JSONArray array = new JSONArray(json);
                        for(int i=0;i<array.length();i++){
                            JSONObject row = array.getJSONObject(i);
                            qr = row.getString("qr");
                            byte[] bytes = BytesUtil.getPrintQRCode(qr,16,2);
                            woyouService.sendRAWData(bytes,callback);
                            woyouService.printText(qr,callback);
                            woyouService.lineWrap(2,callback);
                        }
                    } catch (JSONException e){
                        Log.d(TAG,e.toString());
                    }
                    woyouService.lineWrap(2,callback);
                } catch (RemoteException e){
                    Log.d(TAG,e.toString());
                }
            }
        });
    }

    /**
     * Print Array
     * @param json
     */
    public void printArray(final String json){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    String printType;
                    String printContent;
                    byte[] printBytes = new byte[0];
                    JSONObject options;

                    int fontSize = 20;
                    String fontName = "Courier";
                    int qrcodeModuleSize = 4;
                    int qrcodeErrorLevel = 1;
                    int barcodeSymbol = 4;
                    int barcodeTextPosition = 0;
                    int barcodeHeight = 255;
                    int barcodeWidth = 6;
                    int textAlign = 0;

                    try {
                        JSONArray data = new JSONArray(json);
                        for(int i=0;i<data.length();i++){
                            JSONObject row = data.getJSONObject(i);

                            /* GET DATA OPTIONS FROM ARRAY */
                            options = row.getJSONObject("Options");
                            fontSize = options.getInt("FontSize");
                            fontName = options.getString("FontName");
                            qrcodeModuleSize = options.getInt("QRCodeModuleSize");
                            qrcodeErrorLevel = options.getInt("QRCodeErrorLevel");
                            barcodeSymbol = options.getInt("BarcodeSymbology");
                            barcodeTextPosition = options.getInt("BarcodeTextPosition");
                            barcodeHeight = options.getInt("BarcodeHeight");
                            barcodeWidth = options.getInt("BarcodeWidth");
                            textAlign = options.getInt("TextAlign");

                            printType = row.getString("PrintType");
                            printContent = row.getString("PrintContent");

                            switch (printType){
                                case  "TEXT":
                                    woyouService.printText(printContent,callback);
                                    break;
                                case  "FONTSIZE":
                                    woyouService.setFontSize(fontSize,callback);
                                    break;
                                case  "FONTNAME":
                                    woyouService.setFontName(fontName,callback);
                                    break;
                                case "IMAGE":
                                    printBytes = Base64.decode(printContent,Base64.DEFAULT);
                                    Bitmap bitmapAsset = BitmapFactory.decodeByteArray(printBytes,0,
                                            printBytes.length);
                                    woyouService.printBitmap(bitmapAsset,callback);
                                    break;
                                case "BARCODE":
                                    byte[] barcodeBytes = BytesUtil.getPrintBarCode(printContent,
                                            barcodeSymbol,
                                            barcodeHeight,barcodeWidth,barcodeTextPosition);
                                    woyouService.sendRAWData(barcodeBytes,callback);       /// PRINT QR
                                    break;
                                case "QRCODE":
                                    byte[] qrcodeBytes = BytesUtil.getPrintQRCode(printContent,
                                            qrcodeModuleSize,qrcodeErrorLevel);
                                    woyouService.sendRAWData(qrcodeBytes,callback);       /// PRINT QR
                                    break;
                                case "ENTER":
                                    woyouService.lineWrap(1,callback);
                                    break;
                                case "ALIGN":
                                    woyouService.setAlignment(textAlign,callback);
                                    break;
                            }
                        }

                    } catch (JSONException e){
                        Log.d(TAG,e.toString());
                    }
                } catch (RemoteException e) {
                    Log.d(TAG,e.toString());
                }
            }
        });
    }

    /**
     *
     */

    /// GWK PRINT TEMPLATE
    public void printTemplate01(final String items, final String details){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                String header,date,time,outlet,salesman,amount,paymentType,paymentDue;
                String orderNo,itemCode,itemUoM,itemQty,itemAmount;
                try {
                    woyouService.setAlignment(CENTER_ALIGNMENT,callback);      /// CENTER TEXT               /// IMAGES

                    try {
                        JSONArray itemArr = new JSONArray(items);
                        JSONArray detailArr = new JSONArray(details);

                        Log.d(TAG,"items => "+itemArr.toString());
                        Log.d(TAG,"details => "+detailArr.toString());

                        for(int i=0;i<itemArr.length();i++){
                            JSONObject row = itemArr.getJSONObject(i);
                            Log.d(TAG,"ROW : "+row.toString());
                            Log.d(TAG,"Header : "+row.getString("Header"));
                            header = row.getString("Header");
                            date = row.getString("Date");
                            time = row.getString("Time");
                            outlet = row.getString("Outlet");
                            salesman = row.getString("Salesman");
                            amount = row.getString("Amount");
                            paymentType = row.getString("PaymentType");
                            paymentDue = row.getString("PaymentDue");



                            /// PRINTING
                            woyouService.lineWrap(1,callback);                 /// ENTER
                            woyouService.setAlignment(CENTER_ALIGNMENT,callback);      /// CENTER TEXT

                            /// HEADER
                            woyouService.setFontSize(FONT_HEADER,callback);
                            woyouService.printText(header,callback);
                            woyouService.lineWrap(1,callback);                 /// ENTER

                            woyouService.setFontSize(FONT_BODY,callback);
                            woyouService.printText(LINE_FULL,callback);
                            woyouService.lineWrap(1,callback);                 /// ENTER

                            woyouService.setAlignment(LEFT_ALIGNMENT,callback);      /// LEFT TEXT
                            woyouService.printText("Date         : "+date,callback);
                            woyouService.lineWrap(1,callback);                 /// ENTER

                            woyouService.printText("Time         : "+time,callback);
                            woyouService.lineWrap(1,callback);                 /// ENTER

                            woyouService.printText("Outlet       : "+outlet,callback);
                            woyouService.lineWrap(1,callback);                 /// ENTER

                            woyouService.printText("Salesman     : "+salesman,callback);
                            woyouService.lineWrap(1,callback);                 /// ENTER

                            woyouService.printText(LINE_FULL,callback);
                            woyouService.lineWrap(1,callback);                 /// ENTER

                            woyouService.printText("Amount      : "+amount,callback);
                            woyouService.lineWrap(1,callback);                 /// ENTER
                            woyouService.printText("Payment Type: "+paymentType,callback);
                            woyouService.lineWrap(1,callback);                 /// ENTER
                            woyouService.printText("Payment Due : "+paymentDue,callback);
                            woyouService.lineWrap(1,callback);                 /// ENTER

                            //// ITEM FOREACH
                            for(int j=0;j<detailArr.length();j++){
                                JSONObject rowItem = detailArr.getJSONObject(j);

                                orderNo = rowItem.getString("OrderNo");
                                itemCode = rowItem.getString("Code");
                                itemQty = rowItem.getString("Qty");
                                itemUoM = rowItem.getString("UOM");
                                itemAmount = rowItem.getString("Amount");

                                woyouService.printText("#"+orderNo+"["+itemCode+","+itemUoM+","+itemQty+","+itemAmount+"]",
                                        callback);
                                woyouService.lineWrap(1,callback);                 /// ENTER
                            }

                            /// END ITEM
                            woyouService.printText(LINE_FULL,callback);
                            woyouService.lineWrap(4,callback);                 /// ENTER
                            woyouService.printText("X",callback);
                            woyouService.lineWrap(1,callback);                 /// ENTER
                            woyouService.printText(LINE_FULL,callback);
                            woyouService.printText("Salesman",callback);

                            woyouService.lineWrap(4,callback);                 /// ENTER
                            woyouService.printText("X",callback);
                            woyouService.lineWrap(1,callback);                 /// ENTER
                            woyouService.printText(LINE_FULL,callback);
                            woyouService.printText("Outlet",callback);
                            woyouService.lineWrap(2,callback);                 /// ENTER
                        }
                    } catch (JSONException e){
                        Log.d(TAG,e.toString());
                    }
                    woyouService.lineWrap(2,callback);
                } catch (RemoteException e) {
                    Log.d(TAG,e.toString());
                }
            }
        });
    }

    /**
     * Requested 27 Sept 2019 by Haroon Khalid
     *
     */
    public void printTemplate02(final String lineHeader,final String lineBody,
                                final int fontSizeHeader, final int fontSizeBody,
                                final String header,final String date,final String time,
                                final String outlet, final String salesman, final String amount,
                                final String paymentType, final String paymentDue,
                                final String jsonItems){
        ThreadPoolManager.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                String data;
                try {
                    try {
                        JSONArray itemArr = new JSONArray(jsonItems);

                        /// PRINTING
                        woyouService.lineWrap(1,callback);                 /// ENTER
                        woyouService.setAlignment(CENTER_ALIGNMENT,callback);      /// CENTER TEXT

                        /// HEADER
                        woyouService.setFontSize(fontSizeHeader,callback);
                        woyouService.printText(header,callback);
                        woyouService.lineWrap(1,callback);                 /// ENTER

                        woyouService.setFontSize(fontSizeBody,callback);
                        woyouService.printText(lineHeader,callback);
                        woyouService.lineWrap(1,callback);                 /// ENTER

                        woyouService.setAlignment(LEFT_ALIGNMENT,callback);      /// LEFT TEXT
                        woyouService.printText("DATE:                     "+date,callback);
                        woyouService.lineWrap(1,callback);                 /// ENTER
                        woyouService.printText(lineBody,callback);
                        woyouService.lineWrap(1,callback);

                        woyouService.printText("TIME:                     "+time,callback);
                        woyouService.lineWrap(1,callback);                 /// ENTER
                        woyouService.printText(lineBody,callback);
                        woyouService.lineWrap(1,callback);

                        woyouService.printText("OUTLET:                   "+outlet,callback);
                        woyouService.lineWrap(1,callback);                 /// ENTER
                        woyouService.printText(lineBody,callback);
                        woyouService.lineWrap(1,callback);

                        woyouService.printText("SALESMAN:                 "+salesman,callback);
                        woyouService.lineWrap(1,callback);                 /// ENTER
                        woyouService.printText(lineBody,callback);
                        woyouService.lineWrap(1,callback);

                        woyouService.printText("AMOUNT:                   "+amount,callback);
                        woyouService.lineWrap(1,callback);                 /// ENTER
                        woyouService.printText(lineBody,callback);
                        woyouService.lineWrap(1,callback);

                        woyouService.printText("PAYMENT TYPE:             "+paymentType,callback);
                        woyouService.lineWrap(1,callback);
                        woyouService.printText(lineBody,callback);
                        woyouService.lineWrap(1,callback);

                        woyouService.printText("PAYMENT DUE:              "+paymentDue,callback);
                        woyouService.lineWrap(1,callback);                 /// ENTER
                        woyouService.printText(lineBody,callback);
                        woyouService.lineWrap(1,callback);

                        woyouService.printText("ORDER:                    ",callback);
                        woyouService.lineWrap(1,callback);
                        for(int i=0;i<itemArr.length();i++){
                            JSONObject row = itemArr.getJSONObject(i);
                            data = row.getString("Data");

                            woyouService.printText(data,callback);
                            woyouService.lineWrap(1,callback);
                        }

                        woyouService.printText(lineBody,callback);
                        woyouService.lineWrap(4,callback);
                        woyouService.printText("X",callback);
                        woyouService.lineWrap(1,callback);                 /// ENTER
                        woyouService.printText(lineBody,callback);
                        woyouService.lineWrap(1,callback);
                        woyouService.printText("Salesman",callback);

                        woyouService.lineWrap(5,callback);                 /// ENTER
                        woyouService.printText("X",callback);
                        woyouService.lineWrap(1,callback);                 /// ENTER
                        woyouService.printText(lineBody,callback);
                        woyouService.lineWrap(1,callback);
                        woyouService.printText("Outlet",callback);
                        woyouService.lineWrap(3,callback);                 /// ENTER
                    } catch (JSONException e){
                        Log.d(TAG,e.toString());
                    }
                } catch (RemoteException e) {
                    Log.d(TAG,e.toString());
                }
            }
        });
    }
}
