import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:sunmi_example/blocs/blocs.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:sunmi/sunmi.dart';
import 'package:sunmi_example/blocs/printing_bloc.dart';
import 'package:sunmi_example/common/common.dart';
import 'package:sunmi/class.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:sunmi_example/config.dart' as config;

class AdvancePage extends StatefulWidget {
  @override
  State<AdvancePage> createState() => new AdvancePageState();
}

class AdvancePageState extends State<AdvancePage> {
  List<SunmiPrinter> array = <SunmiPrinter>[];

  @override
  void initState() {
    _generateArray();
    Sunmi.bindPrinter();
    super.initState();
  }

  @override
  void dispose() {
    Sunmi.unbindPrinter();
    super.dispose();
  }

  _launchURL() async {
    const url = config.example_printQRCode;
    if (await canLaunch(url)) {
      await launch(url);
    } else {
      throw 'Could not launch $url';
    }
  }

  void _generateArray() {
    List<dynamic> json = jsonDecode(config.json_printing);
    List<SunmiPrinter> lists = [];
    print(json[0]);
    for (var i = 0; i < json.length; i++) {
      lists.add(new SunmiPrinter.fromJson(json[i]));
    }
    setState(() => array = lists);
  }

  @override
  Widget build(BuildContext context) {
    final bloc = BlocProvider.of<PrintingBloc>(context);

    return new Scaffold(
      appBar: new AppBar(
        title: new Text('PRINT ADVANCE'),
        actions: <Widget>[
          new IconButton(
              icon: new Icon(Icons.code), onPressed: () => _launchURL())
        ],
      ),
      body: new Container(
          padding: EdgeInsets.all(5.0),
          child: new Form(
            child: new ListView(
              children: <Widget>[
                new Container(
                  padding: EdgeInsets.fromLTRB(0.0, 5.0, 0.0, 0.0),
                  child: BaseButton(
                    borderRadius: new BorderRadius.circular(5.0),
                    color: Colors.black,
                    text: new Text(
                      'PRINT',
                      style: TextStyle(color: Colors.white),
                    ),
                    onPressed: () async =>
                        bloc.dispatch(PrintingArray(array: array)),
                  ),
                ),
              ],
            ),
          )),
    );
  }
}
