import 'package:flutter/material.dart';
import 'package:sunmi_example/blocs/blocs.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:sunmi/sunmi.dart';
import 'package:sunmi_example/blocs/printing_bloc.dart';
import 'package:sunmi_example/common/common.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:sunmi/class.dart';
import 'package:sunmi_example/models/core.dart';
import 'package:sunmi_example/models/models.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:sunmi_example/config.dart' as config;

class BarcodePage extends StatefulWidget {
  @override
  State<BarcodePage> createState() => new BarcodePageState();
}

class BarcodePageState extends State<BarcodePage> {
  TextEditingController text = new TextEditingController();
  int selectedWidth = 2;
  int selectedHeight = 50;
  Symbology selectedSymbology =
      new Symbology(symbology: SYMBOLOGY.CODE_128, name: 'CODE 128');
  TextPos selectedTextPos =
      new TextPos(textpos: TEXTPOS.ABOVE_BARCODE, name: 'ABOVE BARCODE');

  List<Symbology> symbology = <Symbology>[];
  List<TextPos> textPos = <TextPos>[];

  @override
  void initState() {
    text.text = '0123456789-QWERTYUIOP';
    symbology
        .add(new Symbology(symbology: SYMBOLOGY.CODE_128, name: 'CODE 128'));
    symbology.add(new Symbology(symbology: SYMBOLOGY.CODE_39, name: 'CODE 39'));
    symbology.add(new Symbology(symbology: SYMBOLOGY.CODE_93, name: 'CODE 93'));
    symbology.add(new Symbology(symbology: SYMBOLOGY.CODEBAR, name: 'CODEBAR'));
    symbology.add(new Symbology(symbology: SYMBOLOGY.EAN_13, name: 'EAN 13'));
    symbology.add(new Symbology(symbology: SYMBOLOGY.EAN_8, name: 'EAN 8'));
    symbology.add(new Symbology(symbology: SYMBOLOGY.ITF, name: 'ITF'));
    symbology.add(new Symbology(symbology: SYMBOLOGY.UPC_A, name: 'UPC A'));
    symbology.add(new Symbology(symbology: SYMBOLOGY.UPC_E, name: 'UPC E'));

    textPos.add(
        new TextPos(textpos: TEXTPOS.ABOVE_BARCODE, name: 'ABOVE BARCODE'));
    textPos.add(
        new TextPos(textpos: TEXTPOS.BELOW_BARCODE, name: 'BELOW BARCODE'));
    textPos.add(new TextPos(textpos: TEXTPOS.BOTH, name: 'BOTH'));
    textPos.add(new TextPos(textpos: TEXTPOS.NO_PRINT_TEXT, name: 'NO TEXT'));

    Sunmi.bindPrinter();
    super.initState();
  }

  @override
  void dispose() {
    Sunmi.unbindPrinter();
    super.dispose();
  }

  _launchURL() async {
    const url = config.example_printQRCode;
    if (await canLaunch(url)) {
      await launch(url);
    } else {
      throw 'Could not launch $url';
    }
  }

  @override
  Widget build(BuildContext context) {
    final bloc = BlocProvider.of<PrintingBloc>(context);

    return new Scaffold(
      appBar: new AppBar(
        title: new Text('PRINT BARCODE'),
        actions: <Widget>[
          new IconButton(
              icon: new Icon(Icons.code), onPressed: () => _launchURL())
        ],
      ),
      body: new Container(
          padding: EdgeInsets.all(5.0),
          child: new Form(
            child: new ListView(
              children: <Widget>[
                new Container(
                    padding: EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 5.0),
                    child: new SizedBox(
                      width: double.infinity,
                      height: 60.0,
                      child: TextFormField(
                        decoration: new InputDecoration(
                          prefixIcon: new Icon(Icons.text_fields,
                              color: Colors.black, size: 18.0),
                          labelText: 'Text',
                          labelStyle: TextStyle(
                              color: Colors.black, fontSize: 10.0, height: 2.0),
                          border: new OutlineInputBorder(),
                        ),
                        controller: text,
                      ),
                    )),
                new Container(
                    padding: EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 5.0),
                    child: selectedSymbology == null
                        ? new TypeAheadField<Symbology>(
                            textFieldConfiguration: TextFieldConfiguration(
                              autofocus: true,
                              autocorrect: false,
                              decoration: InputDecoration(
                                  border: new OutlineInputBorder(),
                                  contentPadding: EdgeInsets.all(5.0),
                                  prefixIcon: new Icon(
                                    Icons.bug_report,
                                    color: Colors.black,
                                    size: 20.0,
                                  ),
                                  suffixIcon: new Icon(Icons.arrow_drop_down,
                                      color: Colors.black),
                                  labelText: 'Symbology',
                                  labelStyle: TextStyle(
                                      color: Colors.black,
                                      fontSize: 10.0,
                                      height: 2.0)),
                            ),
                            loadingBuilder: (context) =>
                                new CircularProgressIndicator(),
                            suggestionsCallback: (String data) async =>
                                symbology
                                    .where((k) =>
                                        k.name.toLowerCase().contains(data))
                                    .toList(),
                            itemBuilder: (context, i) => new ListTile(
                              title: new Text('${i.name}'),
                            ),
                            onSuggestionSelected: (Symbology data) =>
                                setState(() => selectedSymbology = data),
                          )
                        : new ListTile(
                            title: new Text('${selectedSymbology.name}'),
                            trailing: new IconButton(
                              icon: new Icon(
                                Icons.remove_circle,
                                color: Colors.red,
                              ),
                              onPressed: () =>
                                  setState(() => selectedSymbology = null),
                            ),
                          )),
                new Container(
                    padding: EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 5.0),
                    child: selectedTextPos == null
                        ? new TypeAheadField<TextPos>(
                            textFieldConfiguration: TextFieldConfiguration(
                              autofocus: true,
                              autocorrect: false,
                              decoration: InputDecoration(
                                  border: new OutlineInputBorder(),
                                  contentPadding: EdgeInsets.all(5.0),
                                  prefixIcon: new Icon(
                                    Icons.bug_report,
                                    color: Colors.black,
                                    size: 20.0,
                                  ),
                                  suffixIcon: new Icon(Icons.arrow_drop_down,
                                      color: Colors.black),
                                  labelText: 'Text Position',
                                  labelStyle: TextStyle(
                                      color: Colors.black,
                                      fontSize: 10.0,
                                      height: 2.0)),
                            ),
                            loadingBuilder: (context) =>
                                new CircularProgressIndicator(),
                            suggestionsCallback: (String data) async => textPos
                                .where(
                                    (k) => k.name.toLowerCase().contains(data))
                                .toList(),
                            itemBuilder: (context, i) => new ListTile(
                              title: new Text('${i.name}'),
                            ),
                            onSuggestionSelected: (TextPos data) =>
                                setState(() => selectedTextPos = data),
                          )
                        : new ListTile(
                            title: new Text('${selectedTextPos.name}'),
                            trailing: new IconButton(
                              icon: new Icon(
                                Icons.remove_circle,
                                color: Colors.red,
                              ),
                              onPressed: () =>
                                  setState(() => selectedTextPos = null),
                            ),
                          )),
                new Row(
                  children: <Widget>[
                    new Container(
                      padding: EdgeInsets.all(5.0),
                      child: new Text('Width'),
                    ),
                    new Flexible(
                      child: new Container(
                          padding: EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 0.0),
                          child: new Slider(
                            activeColor: Colors.indigoAccent,
                            min: 2,
                            max: 6,
                            onChanged: (data) =>
                                setState(() => selectedWidth = data.toInt()),
                            value: selectedWidth.toDouble(),
                            label: 'Width',
                          )),
                    ),
                    new Container(
                      padding: EdgeInsets.all(5.0),
                      child: new Text('${selectedWidth.toString()}'),
                    )
                  ],
                ),
                new Row(
                  children: <Widget>[
                    new Container(
                      padding: EdgeInsets.all(5.0),
                      child: new Text('Height'),
                    ),
                    new Flexible(
                      child: new Container(
                          padding: EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 0.0),
                          child: new Slider(
                            activeColor: Colors.indigoAccent,
                            min: 1,
                            max: 255,
                            onChanged: (data) =>
                                setState(() => selectedHeight = data.toInt()),
                            value: selectedHeight.toDouble(),
                            label: 'Height',
                          )),
                    ),
                    new Container(
                      padding: EdgeInsets.all(5.0),
                      child: new Text('${selectedHeight.toString()}'),
                    )
                  ],
                ),
                new Divider(),
                new Container(
                  padding: EdgeInsets.fromLTRB(0.0, 5.0, 0.0, 0.0),
                  child: BaseButton(
                      borderRadius: new BorderRadius.circular(5.0),
                      color: Colors.black,
                      text: new Text(
                        'PRINT',
                        style: TextStyle(color: Colors.white),
                      ),
                      onPressed: () => bloc.dispatch(PrintingBarcode(
                          text: text.text,
                          height: selectedHeight,
                          width: selectedWidth,
                          symbology: selectedSymbology.symbology,
                          textpos: selectedTextPos.textpos))),
                ),
              ],
            ),
          )),
    );
  }
}
