import 'package:flutter/material.dart';
import 'package:sunmi_example/blocs/blocs.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:sunmi/sunmi.dart';
import 'package:sunmi_example/blocs/printing_bloc.dart';
import 'package:sunmi_example/common/common.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:flutter/services.dart';
import 'dart:typed_data';
import 'package:sunmi_example/models/core.dart';
import 'package:sunmi_example/models/models.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:sunmi_example/config.dart' as config;

class BitmapPage extends StatefulWidget {
  @override
  State<BitmapPage> createState() => new BitmapPageState();
}

class BitmapPageState extends State<BitmapPage> {
  ImagePrintFile selectedImage = new ImagePrintFile(
      fileUrl: config.defaultPrintLogoImage, filename: 'PRINT-LOGO-GWK.png');
  List<ImagePrintFile> images = <ImagePrintFile>[];

  @override
  void initState() {
    images.add(new ImagePrintFile(
        fileUrl: config.defaultPrintLogoImage, filename: 'PRINT-LOGO-GWK.png'));
    images.add(new ImagePrintFile(
        fileUrl: config.defaultErrorImage, filename: 'ERROR_404.jpg'));
    images.add(new ImagePrintFile(
        fileUrl: config.defaultLogoImage, filename: 'LOGO-GWK.png'));

    Sunmi.bindPrinter();
    super.initState();
  }

  @override
  void dispose() {
    Sunmi.unbindPrinter();
    super.dispose();
  }

  _launchURL() async {
    const url = config.example_printQRCode;
    if (await canLaunch(url)) {
      await launch(url);
    } else {
      throw 'Could not launch $url';
    }
  }

  @override
  Widget build(BuildContext context) {
    final bloc = BlocProvider.of<PrintingBloc>(context);

    return new Scaffold(
      appBar: new AppBar(
        title: new Text('PRINT BITMAP'),
        actions: <Widget>[
          new IconButton(
              icon: new Icon(Icons.code), onPressed: () => _launchURL())
        ],
      ),
      body: new Container(
          padding: EdgeInsets.all(5.0),
          child: new Form(
            child: new ListView(
              children: <Widget>[
                new Container(
                    padding: EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 5.0),
                    child: selectedImage == null
                        ? new TypeAheadField<ImagePrintFile>(
                            textFieldConfiguration: TextFieldConfiguration(
                              autofocus: true,
                              autocorrect: false,
                              decoration: InputDecoration(
                                  border: new OutlineInputBorder(),
                                  contentPadding: EdgeInsets.all(5.0),
                                  prefixIcon: new Icon(
                                    Icons.image,
                                    color: Colors.black,
                                    size: 20.0,
                                  ),
                                  suffixIcon: new Icon(Icons.arrow_drop_down,
                                      color: Colors.black),
                                  labelText: 'Image to Print',
                                  labelStyle: TextStyle(
                                      color: Colors.black,
                                      fontSize: 10.0,
                                      height: 2.0)),
                            ),
                            loadingBuilder: (context) =>
                                new CircularProgressIndicator(),
                            suggestionsCallback: (String data) async => images
                                .where((k) =>
                                    k.filename.toLowerCase().contains(data))
                                .toList(),
                            itemBuilder: (context, i) => new ListTile(
                              title: new Text('${i.filename}'),
                            ),
                            onSuggestionSelected: (ImagePrintFile data) =>
                                setState(() => selectedImage = data),
                          )
                        : new ListTile(
                            title: new Text('${selectedImage.filename}'),
                            trailing: new IconButton(
                              icon: new Icon(
                                Icons.remove_circle,
                                color: Colors.red,
                              ),
                              onPressed: () =>
                                  setState(() => selectedImage = null),
                            ),
                          )),
                new Divider(),
                new Container(
                  padding: EdgeInsets.fromLTRB(0.0, 5.0, 0.0, 0.0),
                  child: BaseButton(
                    borderRadius: new BorderRadius.circular(5.0),
                    color: Colors.black,
                    text: new Text(
                      'PRINT',
                      style: TextStyle(color: Colors.white),
                    ),
                    onPressed: () async {
                      ByteData bytes =
                          await rootBundle.load(selectedImage.fileUrl);
                      Uint8List image = bytes.buffer.asUint8List();
                      bloc.dispatch(PrintingBitmap(bytes: image));
                    },
                  ),
                ),
                selectedImage == null
                    ? new Padding(
                        padding: EdgeInsets.all(0.0),
                      )
                    : new Container(
                        padding: EdgeInsets.fromLTRB(0.0, 5.0, 0.0, 0.0),
                        child: new Image.asset(selectedImage.fileUrl),
                      )
              ],
            ),
          )),
    );
  }
}
