import 'package:flutter/material.dart';
import 'package:sunmi/sunmi.dart';
import 'package:sunmi/class.dart';
import 'dart:convert';


class OrderPay {
  final String orderNo;
  final String code;
  final String uom;
  final String qty;
  final String amount;
  OrderPay({ this.orderNo, this.code, this.uom, this.qty, this.amount });

  Map<String,dynamic> toJson() => {
    'OrderNo':orderNo,
    'Code':code,
    'Qty':qty,
    'UOM':uom,
    'Amount':amount
  };
}

class OrderHeader {
  final String header;
  final String date;
  final String time;
  final String outlet;
  final String salesman;
  final String amount;
  final String paymentType;
  final String paymentDue;

  OrderHeader({ this.header, this.date, this.time, this.outlet, this.salesman, this.amount, this.paymentType, this.paymentDue });

  Map<String,dynamic> toJson() => {
    'Header': header,
    'Date': date,
    'Time': time,
    'Outlet': outlet,
    'Salesman': salesman,
    'Amount': amount,
    'PaymentType': paymentType,
    'PaymentDue': paymentDue
  };
}

class ExamplePage extends StatefulWidget {
  @override
  State<ExamplePage> createState() => new ExamplePageState();
}

class ExamplePageState extends State<ExamplePage> {
  TextEditingController textYouWantToPrint = new TextEditingController();
  List<OrderPay> listsOrder = <OrderPay>[];
  List<OrderHeader> listsHeader = <OrderHeader>[];
  List<SunmiPrinter> array = <SunmiPrinter>[];
  
  @override
  void initState() {
    textYouWantToPrint.text = 'EXAMPLE TEXT YOU WANT TO PRINT';
    Sunmi.bindPrinter(); /// WILL be throw toast, you must wait 2-3 second before done

    listsHeader.add(new OrderHeader(header:'DJAGUNG',date:'2019-01-01',time:'15:00',outlet:'Outlet Name',salesman:'Sales Name S01',amount:'23000',paymentType:'Paytype',paymentDue:'2019-02-01'));

    listsOrder.add(new OrderPay(orderNo:'1',code:'B-12',uom:'Pack',qty:'2',amount:'1000000'));
    listsOrder.add(new OrderPay(orderNo:'2',code:'B-12',uom:'Ball',qty:'1',amount:'1300000'));
    listsOrder.add(new OrderPay(orderNo:'3',code:'B-12',uom:'Pack',qty:'2',amount:'1000000'));

    super.initState();
  }

  @override
  void dispose() {
    Sunmi.bindPrinter();
    textYouWantToPrint.clear();

    /// WILL be throw toast
    super.dispose();
  }

  void examplePrint(){
    final String line = "--------------------------------------";
    List<StringData> data = <StringData>[];
    data.add(new StringData(data:"#1 [Data A, Satuan, Qty, 10000]"));
    data.add(new StringData(data:"#2 [Data B, Satuan, Qty, 20000]"));
    data.add(new StringData(data:"#3 [Data C, Satuan, Qty, 30000]"));
    data.add(new StringData(data:"#4 [Data D, Satuan, Qty, 40000]"));

    Sunmi.template02(
      lineHeader:line,
      lineBody:line,
      header:"DJAGUNG",
      date:"27-09-2019",
      time:"11:23",
      outlet:"OutletName",
      salesman:"SALES NAME",
      amount:"230000",
      paymentType:"NOT DEFINED",
      paymentDue:"30-09-2019",
      orderItems:data
    );
  }

  @override
  Widget build(BuildContext context) {
    return new Scaffold(
      appBar: new AppBar(
        title: new Text('SAMPE PRINT TEXT'),
      ),
      body: new Container(
        child: new Form(
          child: new ListView(
            shrinkWrap: true,
            children: <Widget>[
              new Container(
                  padding: EdgeInsets.all(5.0),
                  child: new SizedBox(
                    width: double.infinity,
                    height: 60.0,
                    child: TextFormField(
                      decoration: new InputDecoration(
                        prefixIcon: new Icon(Icons.text_fields,
                            color: Colors.black, size: 18.0),
                        labelText: 'Add Text Here',
                        labelStyle: TextStyle(
                            color: Colors.black, fontSize: 10.0, height: 2.0),
                        border: new OutlineInputBorder(),
                      ),
                      maxLines: 10,
                      keyboardType: TextInputType.multiline,
                      controller: textYouWantToPrint,
                    ),
                  )),
              new Container(
                padding: EdgeInsets.all(5.0),
                child: new RaisedButton(
                  child: new Text('PRINT'),
                  onPressed: () =>
                      Sunmi.printText(text: textYouWantToPrint.text),
                ),
              ),
              new Container(
                padding: EdgeInsets.all(5.0),
                child: new RaisedButton(
                  child: new Text('EXAMPLE PRINT'),
                  onPressed: () => examplePrint()
                )
              ),
            ],
          ),
        ),
      ),
    );
  }
}
