import 'dart:typed_data';
import 'package:flutter/services.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:sunmi_example/blocs/blocs.dart';
import 'package:sunmi_example/blocs/printing_bloc.dart';
import 'package:sunmi_example/config.dart' as config;
import 'package:flutter/material.dart';
import 'package:sunmi/class.dart';
import 'package:sunmi_example/models/models.dart';
import 'package:sunmi_example/common/common.dart';
import 'package:material_design_icons_flutter/material_design_icons_flutter.dart';
import 'package:sunmi_example/widget/qrcode_page.dart';
import 'package:sunmi_example/widget/widget.dart';

class PrintingPage extends StatefulWidget {
  @override
  State<PrintingPage> createState() => PrintingPageState();
}

class PrintingPageState extends State<PrintingPage> {
  List<SunmiPrinter> array = <SunmiPrinter>[];
  List<MenuService> menus = <MenuService>[];
  Uint8List bytes;

  /// additionalprinting
  TextEditingController text = new TextEditingController();
  TextEditingController fontSize = new TextEditingController();
  TextEditingController fontName = new TextEditingController();

  @override
  void initState() {
    super.initState();
    _getBytesImage();
    _generateMenu();
  }

  @override
  void dispose() {
    super.dispose();
  }

  void _getBytesImage() async {
    ByteData getBytes = await rootBundle.load(config.defaultPrintLogoImage);
    Uint8List image = getBytes.buffer.asUint8List();
    setState(() => bytes = image);
  }

  void _generateMenu() {
    menus.add(
      new MenuService(
        name: 'TEXT',
        icons: Icon(Icons.text_fields),
        onTap: () => Navigator.push(
            context,
            new MaterialPageRoute(
                fullscreenDialog: true,
                builder: (BuildContext context) => new TextPage())),
      ),
    );

    menus.add(
      new MenuService(
        name: 'BARCODE',
        icons: Icon(MdiIcons.barcode),
        onTap: () => Navigator.push(
            context,
            new MaterialPageRoute(
                fullscreenDialog: true,
                builder: (BuildContext context) => new BarcodePage())),
      ),
    );

    menus.add(
      new MenuService(
        name: 'QRCODE',
        icons: Icon(MdiIcons.qrcode),
        onTap: () => Navigator.push(
            context,
            new MaterialPageRoute(
                fullscreenDialog: true,
                builder: (BuildContext context) => new QRCodePage())),
      ),
    );

    menus.add(
      new MenuService(
        name: 'UTILS',
        icons: Icon(Icons.line_style),
        onTap: () => Navigator.push(
            context,
            new MaterialPageRoute(
                fullscreenDialog: true,
                builder: (BuildContext context) => new UtilsPage())),
      ),
    );

    menus.add(
      new MenuService(
        name: 'ADVANCE',
        icons: Icon(MdiIcons.codeArray),
        onTap: () => Navigator.push(
            context,
            new MaterialPageRoute(
                fullscreenDialog: true,
                builder: (BuildContext context) => new AdvancePage())),
      ),
    );

    menus.add(
      new MenuService(
        name: 'BITMAP',
        icons: Icon(MdiIcons.image),
        onTap: () => Navigator.push(
            context,
            new MaterialPageRoute(
                fullscreenDialog: true,
                builder: (BuildContext context) => new BitmapPage())),
      ),
    );

    menus.add(
      new MenuService(
        name: 'EXAMPLE REQUEST',
        icons: Icon(MdiIcons.musicRestQuarter),
        onTap: () => Navigator.push(
            context,
            new MaterialPageRoute(
                fullscreenDialog: true,
                builder: (BuildContext context) => new ExamplePage())),
      ),
    );
  }

  void _openAlert({String error}) {
    showDialog(
        context: context,
        builder: (context) => new AlertDialog(
              title: new Text('Alert'),
              content: new Text(error),
            ));
  }

  @override
  Widget build(BuildContext context) {
    return new Scaffold(
      appBar: new AppBar(
        title: new Text('PRINT EXAMPLE'),
      ),
      body: new Container(
        child: BlocListener<PrintingBloc, PrintingState>(
          listener: (context, state) {
            if (state is ErrorPrinting) {
              _openAlert(error: state.error);
            }
          },
          child: BlocBuilder<PrintingBloc, PrintingState>(
            builder: (context, state) {
              if (state is LoadingPrinting) {
                return new LoadingIndicator();
              }
              return new Container(
                child: new MenuServiceState(
                  items: menus,
                ),
              );
            },
          ),
        ),
      ),
    );
  }
}

class MenuServiceState extends StatelessWidget {
  final List<MenuService> items;
  final int crossAxis;
  MenuServiceState({this.items, this.crossAxis = 3});

  @override
  Widget build(BuildContext context) {
    return new Container(
      margin: EdgeInsets.only(top: 8.0, bottom: 8.0),
      child: new GridView.builder(
        shrinkWrap: true,
        gridDelegate: new SliverGridDelegateWithFixedCrossAxisCount(
            crossAxisCount: crossAxis),
        physics: ClampingScrollPhysics(),
        itemCount: items.length,
        itemBuilder: (context, i) => new GestureDetector(
          onTap: items[i].onTap,
          child: new Container(
            alignment: Alignment.center,
            child: new Column(
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                new Container(
                  decoration: new BoxDecoration(
                      border: Border.all(color: Colors.grey, width: 1.0),
                      borderRadius:
                          new BorderRadius.all(new Radius.circular(20.0))),
                  padding: EdgeInsets.all(12.0),
                  child: items[i].icons,
                ),
                new Padding(
                  padding: EdgeInsets.only(top: 6.0),
                ),
                new Text("${items[i].name}",textAlign:TextAlign.center,)
              ],
            ),
          ),
        ),
      ),
    );
  }
}
