import 'package:flutter/material.dart';
import 'package:qr_flutter/qr_flutter.dart';
import 'package:sunmi_example/blocs/blocs.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:sunmi/sunmi.dart';
import 'package:sunmi_example/blocs/printing_bloc.dart';
import 'package:sunmi_example/common/common.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:sunmi/class.dart';
import 'package:sunmi_example/models/core.dart';
import 'package:sunmi_example/models/models.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:sunmi_example/config.dart' as config;

class QRCodePage extends StatefulWidget {
  @override
  State<QRCodePage> createState() => new QRCodePageState();
}

class QRCodePageState extends State<QRCodePage> {
  TextEditingController text = new TextEditingController();
  int selectedModuleSize = 16;
  ErrorLevel selectedErrorLevel =
      new ErrorLevel(errorlevel: ERRORLEVEL.M, name: 'M');
  List<ErrorLevel> errorLevel = <ErrorLevel>[];

  @override
  void initState() {
    text.text = '0123456789-QWERTYUIOP';
    errorLevel.add(new ErrorLevel(errorlevel: ERRORLEVEL.H, name: 'H'));
    errorLevel.add(new ErrorLevel(errorlevel: ERRORLEVEL.M, name: 'M'));
    errorLevel.add(new ErrorLevel(errorlevel: ERRORLEVEL.L, name: 'L'));
    errorLevel.add(new ErrorLevel(errorlevel: ERRORLEVEL.Q, name: 'Q'));
    Sunmi.bindPrinter();
    super.initState();
  }

  @override
  void dispose() {
    Sunmi.unbindPrinter();
    super.dispose();
  }

  _launchURL() async {
    const url = config.example_printQRCode;
    if (await canLaunch(url)) {
      await launch(url);
    } else {
      throw 'Could not launch $url';
    }
  }

  @override
  Widget build(BuildContext context) {
    final bloc = BlocProvider.of<PrintingBloc>(context);

    return new Scaffold(
      appBar: new AppBar(
        title: new Text('PRINT QRCODE'),
        actions: <Widget>[
          new IconButton(
              icon: new Icon(Icons.code), onPressed: () => _launchURL())
        ],
      ),
      body: new Container(
          padding: EdgeInsets.all(5.0),
          child: new Form(
            child: new ListView(
              children: <Widget>[
                new Container(
                    padding: EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 5.0),
                    child: new SizedBox(
                      width: double.infinity,
                      height: 60.0,
                      child: TextFormField(
                        decoration: new InputDecoration(
                          prefixIcon: new Icon(Icons.text_fields,
                              color: Colors.black, size: 18.0),
                          labelText: 'Text',
                          labelStyle: TextStyle(
                              color: Colors.black, fontSize: 10.0, height: 2.0),
                          border: new OutlineInputBorder(),
                        ),
                        controller: text,
                      ),
                    )),
                new Container(
                    padding: EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 5.0),
                    child: selectedErrorLevel == null
                        ? new TypeAheadField<ErrorLevel>(
                            textFieldConfiguration: TextFieldConfiguration(
                              autofocus: true,
                              autocorrect: false,
                              decoration: InputDecoration(
                                  border: new OutlineInputBorder(),
                                  contentPadding: EdgeInsets.all(5.0),
                                  prefixIcon: new Icon(
                                    Icons.bug_report,
                                    color: Colors.black,
                                    size: 20.0,
                                  ),
                                  suffixIcon: new Icon(Icons.arrow_drop_down,
                                      color: Colors.black),
                                  labelText: 'Error Level',
                                  labelStyle: TextStyle(
                                      color: Colors.black,
                                      fontSize: 10.0,
                                      height: 2.0)),
                            ),
                            loadingBuilder: (context) =>
                                new CircularProgressIndicator(),
                            suggestionsCallback: (String data) async =>
                                errorLevel,
                            itemBuilder: (context, i) => new ListTile(
                              title: new Text('ERROR LEVEL ${i.name}'),
                            ),
                            onSuggestionSelected: (ErrorLevel data) =>
                                setState(() => selectedErrorLevel = data),
                          )
                        : new ListTile(
                            title: new Text(
                                'ERROR LEVEL ${selectedErrorLevel.name}'),
                            trailing: new IconButton(
                              icon: new Icon(
                                Icons.remove_circle,
                                color: Colors.red,
                              ),
                              onPressed: () =>
                                  setState(() => selectedErrorLevel = null),
                            ),
                          )),
                new Row(
                  children: <Widget>[
                    new Container(
                      padding: EdgeInsets.all(5.0),
                      child: new Text('Size'),
                    ),
                    new Flexible(
                      child: new Container(
                          padding: EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 0.0),
                          child: new Slider(
                            activeColor: Colors.indigoAccent,
                            min: 4,
                            max: 100,
                            onChanged: (data) => setState(
                                () => selectedModuleSize = data.toInt()),
                            value: selectedModuleSize.toDouble(),
                            label: 'Module Size',
                          )),
                    ),
                    new Container(
                      padding: EdgeInsets.all(5.0),
                      child: new Text('${selectedModuleSize.toString()}'),
                    )
                  ],
                ),
                new Divider(),
                new Container(
                  padding: EdgeInsets.fromLTRB(0.0, 5.0, 0.0, 0.0),
                  child: BaseButton(
                    borderRadius: new BorderRadius.circular(5.0),
                    color: Colors.black,
                    text: new Text(
                      'PRINT',
                      style: TextStyle(color: Colors.white),
                    ),
                    onPressed: () => bloc.dispatch(PrintingQRCode(
                        text: text.text,
                        moduleSize: selectedModuleSize,
                        errorlevel: selectedErrorLevel.errorlevel)),
                  ),
                ),
                new Container(
                  alignment: Alignment.center,
                  child: new QrImage(
                    data: text.text,
                    size: 300.0,
                  ),
                )
              ],
            ),
          )),
    );
  }
}
