import 'package:flutter/material.dart';
import 'package:sunmi_example/blocs/blocs.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:sunmi/sunmi.dart';
import 'package:sunmi_example/common/common.dart';
import 'package:sunmi_example/config.dart' as config;
import 'package:url_launcher/url_launcher.dart';

class TextPage extends StatefulWidget {
  @override
  State<TextPage> createState() => new TextState();
}

class TextState extends State<TextPage> {
  TextEditingController text = new TextEditingController();
  TextEditingController fontSize = new TextEditingController();
  TextEditingController fontName = new TextEditingController();

  @override
  void initState() {
    text.text = '''
      0123456789-QWERTYUIOP
      0123456789-QWERTYUIOP
      0123456789-QWERTYUIOP
      0123456789-QWERTYUIOP
      0123456789-QWERTYUIOP
    ''';
    Sunmi.bindPrinter();
    super.initState();
  }

  @override
  void dispose() {
    Sunmi.unbindPrinter();
    super.dispose();
  }

  _launchURL() async {
    const url = config.example_printText;
    if (await canLaunch(url)) {
      await launch(url);
    } else {
      throw 'Could not launch $url';
    }
  }

  @override
  Widget build(BuildContext context) {
    final bloc = BlocProvider.of<PrintingBloc>(context);

    return new Scaffold(
      appBar: new AppBar(
        title: new Text('PRINT TEXT'),
        actions: <Widget>[
          new IconButton(
              icon: new Icon(Icons.code), onPressed: () => _launchURL())
        ],
      ),
      body: new Container(
          padding: EdgeInsets.all(5.0),
          child: new Form(
            child: new ListView(
              children: <Widget>[
                new Container(
                    padding: EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 5.0),
                    child: new SizedBox(
                      width: double.infinity,
                      height: 60.0,
                      child: TextFormField(
                        decoration: new InputDecoration(
                          prefixIcon: new Icon(Icons.text_fields,
                              color: Colors.black, size: 18.0),
                          labelText: 'Text',
                          labelStyle: TextStyle(
                              color: Colors.black, fontSize: 10.0, height: 2.0),
                          border: new OutlineInputBorder(),
                        ),
                        maxLines: 10,
                        keyboardType: TextInputType.multiline,
                        controller: text,
                      ),
                    )),
                // new Container(
                //   padding: EdgeInsets.fromLTRB(0.0,0.0,0.0,5.0),
                //   child: new SizedBox(
                //     width: double.infinity,
                //     height: 60.0,
                //     child: TextFormField(
                //       decoration: new InputDecoration(
                //         prefixIcon: new Icon(Icons.font_download,color:Colors.black,size:18.0),
                //         labelText: 'Font Name',
                //         labelStyle: TextStyle(color:Colors.black,fontSize:10.0,height:2.0),
                //         border: new OutlineInputBorder(),
                //       ),
                //       controller: fontName,
                //     ),
                //   )
                // ),
                // new Container(
                //   padding: EdgeInsets.fromLTRB(0.0,0.0,0.0,0.0),
                //   child: new SizedBox(
                //     width: double.infinity,
                //     height: 60.0,
                //     child: TextFormField(
                //       decoration: new InputDecoration(
                //         prefixIcon: new Icon(Icons.art_track,color:Colors.black,size:18.0),
                //         labelText: 'Font Size',
                //         labelStyle: TextStyle(color:Colors.black,fontSize:10.0,height:2.0),
                //         border: new OutlineInputBorder(),
                //       ),
                //       controller: fontSize,
                //     ),
                //   )
                // ),
                new Divider(),
                new Container(
                  padding: EdgeInsets.fromLTRB(0.0, 5.0, 0.0, 0.0),
                  child: BaseButton(
                    borderRadius: new BorderRadius.circular(5.0),
                    color: Colors.black,
                    text: new Text(
                      'PRINT',
                      style: TextStyle(color: Colors.white),
                    ),
                    onPressed: () =>
                        bloc.dispatch(PrintingText(text: text.text)),
                  ),
                )
              ],
            ),
          )),
    );
  }
}
