import 'dart:async';
import 'dart:convert';
import 'package:sunmi/class.dart';
import 'package:flutter/services.dart';
import 'dart:typed_data';

class Sunmi {
  static const MethodChannel _c = const MethodChannel('sunmi');

  static Future<String> get platformVersion async => await _c.invokeMethod('getPlatformVersion');
  static Future<String> get getImei async => await _c.invokeMethod('GetUUID');

  /// SUNMI PRINTER
  static Future<Null> bindPrinter() async { await _c.invokeMethod("bindPrinter"); }
  static Future<Null> unbindPrinter() async { await _c.invokeMethod("unbindPrinter"); }
  static Future<Null> selfCheckingPrinter() async { await _c.invokeMethod("selfCheckingPrinter"); }
  static Future<SunmiProperties> getProperties() async { 
    Map<dynamic,dynamic> json = await _c.invokeMethod("getProperties");
    return new SunmiProperties.fromJson(json);
  }

  static Future<Null> printText({ String text }) async {
    Map<String,dynamic> args = <String,dynamic>{};
    args.putIfAbsent("text", () => text);
    await _c.invokeMethod("printText",args);
  }

  static Future<Null> printFreeTextWithFont({ String text, String fontName, int fontSize }) async {
    Map<String,dynamic> args = <String,dynamic>{};
    args.putIfAbsent("text", () => text);
    args.putIfAbsent("fontName", () => fontName);
    args.putIfAbsent("fontSize", () => fontSize);
    await _c.invokeMethod("printTextWithFont",args);
  }

  static Future<Null> printBitmap({ Uint8List bitmap }) async {
    Map<String,dynamic> args = <String,dynamic>{};
    args.putIfAbsent("bitmap", () => bitmap);
    await _c.invokeMethod("printBitmap",args);
  }

  static Future<Null> printQRCode({ String text, int moduleSize, ERRORLEVEL errorLevel }) async {
    Map<String,dynamic> args = <String,dynamic>{};
    args.putIfAbsent("text", () => text);
    args.putIfAbsent("moduleSize", () => moduleSize > 16 ? 16 : moduleSize < 4 ? 4 : 4);
    args.putIfAbsent("errorLevel", () => errorLevel == ERRORLEVEL.L ? 0
      : errorLevel == ERRORLEVEL.M ? 1
      : errorLevel == ERRORLEVEL.Q ? 2
      : errorLevel == ERRORLEVEL.H ? 3
      : 0);
    await _c.invokeMethod("printQRCode",args);
  }

  static Future<Null> printBarcode({ String text, SYMBOLOGY symbology, int height, int width, TEXTPOS textPosition }) async {
    Map<String,dynamic> args = <String,dynamic>{};

    args.putIfAbsent("text", () => text);
    args.putIfAbsent("symbology", () => symbology == SYMBOLOGY.UPC_A ? 0
      : symbology == SYMBOLOGY.UPC_E ? 1
      : symbology == SYMBOLOGY.EAN_13 ? 2
      : symbology == SYMBOLOGY.EAN_8 ? 3
      : symbology == SYMBOLOGY.CODE_39 ? 4
      : symbology == SYMBOLOGY.ITF ? 5 
      : symbology == SYMBOLOGY.CODEBAR ? 6
      : symbology == SYMBOLOGY.CODE_93 ? 7
      : symbology == SYMBOLOGY.CODE_128 ? 8 
      : 4);
    args.putIfAbsent("height", () => height > 255 ? 255 : height < 1 ? 1 : height);
    args.putIfAbsent("width", () => width > 6 ? 6 : width < 2 ? 2 : width);
    args.putIfAbsent("textPosition", () => textPosition == TEXTPOS.NO_PRINT_TEXT ? 0 
      : textPosition == TEXTPOS.ABOVE_BARCODE ? 1 
      : textPosition == TEXTPOS.BELOW_BARCODE ? 2
      : textPosition == TEXTPOS.BOTH ? 3 
      : 0);
    await _c.invokeMethod("printBarcode",args);
  }

  static Future<Null> setLineWrap({ int line=1 }) async {
    Map<String,dynamic> args = <String,dynamic>{};
    args.putIfAbsent("line", () => line);
    await _c.invokeMethod("setLineWrap",args);
  }

  static Future<void> printArray({ List<SunmiPrinter> array }) async {
    Map<String,dynamic> args = <String,dynamic>{};
    args.putIfAbsent("json", () => jsonEncode(array));
    await _c.invokeMethod("printArray",args);
  }

  /// PRINT QR WITH QRCODE
  // static Future<void> template({ String jsonItems, String jsonDetails }) async {
  //   Map<String,dynamic> args = <String,dynamic>{};
  //   args.putIfAbsent("jsonItems", () => jsonItems);
  //   args.putIfAbsent("jsonDetails", () => jsonDetails);

  //   await _c.invokeMethod("template01",args);
  // }

  /// REQUESTED BY HAROON KHALID
  static Future<void> template02({ 
    String lineHeader="",
    String lineBody="",
    int fontSizeHeader,
    int fontSizeBody,
    String header="",
    String date="",
    String time="",
    String outlet="",
    String salesman="",
    String amount="",
    String paymentType="",
    String paymentDue="",
    List<StringData> orderItems,
  }) async {
    Map<String,dynamic> args = <String,dynamic>{};
    args.putIfAbsent("lineHeader", () => lineHeader);
    args.putIfAbsent("lineBody", () => lineBody);
    args.putIfAbsent("fontSizeHeader", () => fontSizeHeader == null ? 30 : fontSizeHeader);
    args.putIfAbsent("fontSizeBody", () => fontSizeBody == null ? 20 : fontSizeBody);
    args.putIfAbsent("header", () => header);
    args.putIfAbsent("date", () => date);
    args.putIfAbsent("time", () => time);
    args.putIfAbsent("outlet", () => outlet);
    args.putIfAbsent("salesman", () => salesman);
    args.putIfAbsent("amount", () => amount);
    args.putIfAbsent("paymentType", () => paymentType);
    args.putIfAbsent("paymentDue", () => paymentDue);
    args.putIfAbsent("jsonItems", () => jsonEncode(orderItems));

    await _c.invokeMethod("template02",args);
  }
}
